//******************************************************************************
//GoldMap
//by BlueSonic (aka Jigglypuff)
//******************************************************************************
unit Events;

interface

uses
Windows, Classes, IniFiles, Graphics, Pngimage, Sysutils, Dialogs;

type

//warp data structure
TWarp = packed record 
  Y: Byte;
  X: Byte;
  WarpTo: Byte;
  MapBank: Byte;
  Map: Byte;
  end;

//trigger data structure
TTrigger = packed record
  Unknown1: Byte;
  Y: Byte;
  X: Byte;
  Unknown2: array [1..5] of Byte;
  end;

//signpost data
TSignpost = packed record
Y: Byte;
X: Byte;
Func: Byte;
TextBlock: Byte;
TextBank: Byte;
end;

//people data
TPeople = packed record  
Pict: Byte;
Y: Byte;
X: Byte;
Face: Byte;
Move: Byte;
Unk1: Byte;
Unk2: Byte;
Func: Byte;
Unk3: Byte;
TextBlock: Byte;
TextBank: Byte;
When: Byte;
Hide: Byte;
end;

//loads events from specified address (address is found from MapHeader2)
procedure LoadEvents(Addr: Integer);
//draws the events on canvas
procedure DrawEvents(Canvas: TCanvas);
//saves events to rom
procedure SaveEvents;
//procedure that draws the event picture
procedure DrawBox(Canvas: TCanvas; X,Y,PIC: Integer);

const
EWarp = 0;
ETrigger = 1;
ESignPost = 2;
EPeople = 3;

var
NumOfWarp,NumOfTrigger,NumOfSignpost,NumOfPeople: Byte;
EventsAddr: Integer;
Warps: array [1..64] of TWarp;
Triggers: array [1..64] of TTrigger;
Signposts: array [1..64] of TSignpost;
Peoples: array [1..64] of TPeople;

implementation

uses Main;

procedure LoadEvents(Addr: Integer);
var
ZeroWord: Word;
I: Integer;
LoadEvents: boolean;
begin
EventsAddr := Addr;
LoadEvents := True;
Rom.Position := Addr;
Rom.Read(ZeroWord,2);
if ZeroWord <> $0000 then
 begin
 MessageDlg('There seems be error in event data. Events will not be loaded.',mtWarning,[mbOK],0);
 LoadEvents := False;
 end;
if LoadEvents then
 begin
 Rom.Read(NumOfWarp,1);
 Rom.Read(Warps,SizeOf(TWarp) * NumOfWarp);
 Rom.Read(NumOfTrigger,1);
 Rom.Read(Triggers,SizeOf(TTrigger) * NumOfTrigger);
 Rom.Read(NumOfSignpost,1);
 Rom.Read(Signposts,SizeOf(TSignpost) * NumOfSignpost);
 Rom.Read(NumOfPeople,1);
 Rom.Read(Peoples,SizeOf(TPeople) * NumOfPeople);
{//Load event information
WarpStart := EventsIni.ReadInteger(MapName,'Warp offset',0);
NumOfWarp := EventsIni.ReadInteger(MapName,'Warps',0);
NumOfSingpost := EventsIni.ReadInteger(MapName,'Singposts',0);
SingpostStart := EventsIni.ReadInteger(MapName,'Singpost offset',0);
NumOfPeople := EventsIni.ReadInteger(MapName,'Peoples',0);
PeopleStart := EventsIni.ReadInteger(MapName,'People offset',0);
//Read events from rom
Rom.Position := WarpStart;
Rom.Read(Warps,SizeOf(TWarp) * NumOfWarp);
Rom.Position := SingPostStart;
Rom.Read(Singposts,SizeOf(TSingpost) * NumOfSingpost);
Rom.Position := PeopleStart;
Rom.Read(Peoples,SizeOf(TPeople) * NumOfPeople);}
 end
else
 begin
 EventsAddr := 0;
 end;

//if map has no events, hide the event scroll box, and show the label telling there's no events
if not LoadEvents then
 begin
 MainForm.EventScrollBox.Hide;
 MainForm.NoEventsLabel.Show;
 MainForm.ReBuildEvents1.Enabled := False;
 end
else
 begin
 MainForm.EventScrollBox.Show;
 MainForm.NoEventsLabel.Hide;
 MainForm.ReBuildEvents1.Enabled := True;
 end;

//Decrease people coordinates
for I := 1 to NumOfPeople do
 begin
 if Peoples[I].X >= 4 then
 Peoples[I].X := Peoples[I].X - 4
 else
 MainForm.EventBox.Hint := 'Warning: Event loader detected corrupted people event data.';
 if Peoples[I].Y >= 4 then
 Peoples[I].Y := Peoples[I].Y - 4
 else
 MainForm.EventBox.Hint := 'Warning: Event loader detected corrupted people event data.';
 end;

//set right size for event box
with MainForm do
 begin
 EventBox.Height := MapBox.Height;
 EventBox.width := MapBox.width;
 end;
//show events
DrawEvents(MainForm.EventBox.Canvas);
end;

//procedure that draws the event picture
procedure DrawBox(Canvas: TCanvas; X,Y,PIC: Integer);
begin
 case PIC of
 EWarp: TPNGObject(MainForm.WarpImage.Picture.Graphic).Draw(Canvas,Rect(X,Y,X+32,Y+32));
 ETrigger: TPNGObject(MainForm.TriggerImage.Picture.Graphic).Draw(Canvas,Rect(X,Y,X+32,Y+32));
 ESignPost: TPNGObject(MainForm.SignpostImage.Picture.Graphic).Draw(Canvas,Rect(X,Y,X+32,Y+32));
 EPeople: TPNGObject(MainForm.PeopleImage.Picture.Graphic).Draw(Canvas,Rect(X,Y,X+32,Y+32));
 end;
end;

procedure DrawEvents(Canvas: TCanvas);
var
I: Integer;
X,Y: Integer;
begin
//redraw map image
MainForm.DrawMap(Canvas);
//draws events to EventBox
for Y := Canvas.ClipRect.Top  div 32 to ((Canvas.ClipRect.Bottom-1) div 32) do
for X := Canvas.ClipRect.Left div 32 to ((Canvas.ClipRect.Right-1)  div 32) do
 begin
 for I := 1 to NumOfWarp do
 if (Warps[I].X = X) and (Warps[I].Y = Y) then
 DrawBox(Canvas,X*32,Y*32,EWarp);
 for I := 1 to NumOfTrigger do
 if (Triggers[I].X = X) and (Triggers[I].Y = Y) then
 DrawBox(Canvas,X*32,Y*32,ETrigger);
 for I := 1 to NumOfSignpost do
 if (Signposts[I].X = X) and (Signposts[I].Y = Y) then
 DrawBox(Canvas,Signposts[I].X*32,Signposts[I].Y*32,ESignPost);
 for I := 1 to NumOfPeople do
 if (Peoples[I].X = X) and (Peoples[I].Y = Y) then
 DrawBox(Canvas,Peoples[I].X*32,Peoples[I].Y*32,EPeople);
 end;
end;

procedure SaveEvents;
var
I: Integer;
ZeroWord: Word;
begin
//if there's no events there's nothing to save
if EventsAddr = 0 then Exit;
//increase people coordinates
for I := 1 to NumOfPeople do begin
Peoples[I].X := Peoples[I].X + 4;
Peoples[I].Y := Peoples[I].Y + 4;
end;
ZeroWord := 0;
//write events
Rom.Position := EventsAddr;
Rom.Write(ZeroWord,2);
Rom.Write(NumOfWarp,1);
Rom.write(Warps,SizeOf(TWarp)*NumOfWarp);
Rom.Write(NumOfTrigger,1);
Rom.Write(Triggers,SizeOf(TTrigger) * NumOfTrigger);
Rom.Write(NumOfSignpost,1);
Rom.write(Signposts,SizeOf(TSignpost)*NumOfSignpost);
Rom.Write(NumOfPeople,1);
Rom.write(Peoples,SizeOf(TPeople)*NumOfPeople);
//events need to be decreased again
for I := 1 to NumOfPeople do
 begin
 Peoples[I].X := Peoples[I].X - 4;
 Peoples[I].Y := Peoples[I].Y - 4;
 end;
end;

end.
